__version__ = '0.1.1'

__author__ = 'Yves Chevallier'
__description__ = 'Functional tests for teaching activity'
__email__ = 'yves.chevallier@heig-vd.ch'
__license__ = 'MIT'
__copyright__ = 'Copyright 2020 HEIG-VD'

from .validation import Test, Tests
from .test import TestCase, TestSuite
from .executable import Executable
